"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initProjectCommand = void 0;
const core_1 = require("@takomo/core");
const util_1 = require("@takomo/util");
const joi_1 = __importDefault(require("joi"));
const path_1 = __importDefault(require("path"));
const samples_1 = require("./samples");
const schema = joi_1.default.object({
    project: core_1.project,
    regions: joi_1.default.array().items(core_1.region).unique(),
    createSamples: joi_1.default.boolean(),
}).unknown(true);
const pathsToDirectories = (projectDir) => [
    core_1.Constants.STACKS_DIR,
    core_1.Constants.TEMPLATES_DIR,
    core_1.Constants.RESOLVERS_DIR,
    core_1.Constants.HOOKS_DIR,
    core_1.Constants.PARTIALS_DIR,
    core_1.Constants.HELPERS_DIR,
].map((d) => path_1.default.join(projectDir, d));
const checkForExistingDirs = async (projectDir, dirs) => {
    const existingDirs = [];
    for (const dir of dirs) {
        if (await util_1.dirExists(dir)) {
            existingDirs.push(dir);
        }
    }
    if (existingDirs.length > 0) {
        const existingDirsString = existingDirs
            .map((d) => `  - ${d.split(path_1.default.sep).pop()}`)
            .join("\n");
        throw new util_1.TakomoError(`Could not initialize a new project in directory: ${projectDir}. ` +
            "Following directories already exists in the target directory:\n\n" +
            existingDirsString);
    }
};
const createDirs = async (dirs) => {
    await Promise.all(dirs.map(async (d) => util_1.createDir(d)));
};
const createRootStackConfigFile = async (projectDir, info) => {
    const rootStackGroupFile = path_1.default.join(projectDir, core_1.Constants.STACKS_DIR, core_1.Constants.STACK_GROUP_CONFIG_FILE);
    let contents = "";
    if (info.project) {
        contents += `project: ${info.project}\n`;
    }
    if (info.regions.length === 1) {
        contents += `regions: ${info.regions[0]}\n`;
    }
    else if (info.regions.length > 1) {
        contents += `regions:\n`;
        info.regions.forEach((r) => (contents += `  - ${r}\n`));
    }
    await util_1.createFile(rootStackGroupFile, contents);
};
const createSamples = async (projectDir, info) => {
    if (!info.createSamples) {
        return;
    }
    return samples_1.createSampleFiles(projectDir);
};
const createDescription = (projectDir, info) => info.createSamples
    ? ".\n" +
        "├─ stacks            # dir for stack configuration files\n" +
        "│  ├- vpc.yml        # configuration file for sample vpc stack\n" +
        "│  └─ config.yml     # root stack group configuration file\n" +
        "├─ templates         # dir for CloudFormation template files\n" +
        "│  └─ vpc.yml        # CloudFormation template for sample vpc stack\n" +
        "├─ helpers           # dir for custom Handlebars helpers\n" +
        "├─ partials          # dir for custom Handlebars partials\n" +
        "├─ resolvers         # dir for custom resolvers\n" +
        "└─ hooks             # dir for custom hooks\n"
    : ".\n" +
        "├─ stacks            # dir for stack configuration files\n" +
        "│  └─ config.yml     # root stack group configuration file\n" +
        "├─ templates         # dir for CloudFormation template files\n" +
        "├─ helpers           # dir for custom Handlebars helpers\n" +
        "├─ partials          # dir for custom Handlebars partials\n" +
        "├─ resolvers         # dir for custom resolvers\n" +
        "└─ hooks             # dir for custom hooks\n";
const initProject = async (input, io) => {
    const { options } = input;
    const projectDir = options.getProjectDir();
    const dirs = pathsToDirectories(projectDir);
    await checkForExistingDirs(projectDir, dirs);
    const info = await io.promptProjectInformation(input);
    await createDirs(dirs);
    await createRootStackConfigFile(projectDir, info);
    await createSamples(projectDir, info);
    const description = createDescription(projectDir, info);
    return {
        projectDir,
        description,
        message: "Success",
        status: core_1.CommandStatus.SUCCESS,
        success: true,
        watch: input.watch.stop(),
    };
};
exports.initProjectCommand = async (input, io) => util_1.validateInput(schema, input)
    .then((input) => initProject(input, io))
    .then(io.printOutput);
//# sourceMappingURL=command.js.map